//-----------------------------------------------------------------------------
// File : BasicPS.hlsl
// Desc : Pixel Shader.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "BRDF.hlsli"

#ifndef MIN_DIST
#define MIN_DIST (0.01)
#endif//MIN_DIST


///////////////////////////////////////////////////////////////////////////////
// VSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct VSOutput
{
    float4      Position        : SV_POSITION;          // ʒuWł.
    float2      TexCoord        : TEXCOORD;             // eNX`Wł.
    float3      WorldPos        : WORLD_POS;            // [hԂ̈ʒuWł.
    float3x3    InvTangentBasis : INV_TANGENT_BASIS;    // ڐԂւ̊ϊs̋tsł.
};

///////////////////////////////////////////////////////////////////////////////
// PSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct PSOutput
{
    float4  Color : SV_TARGET0;     // o̓J[ł.
};

///////////////////////////////////////////////////////////////////////////////
// CbLight constant buffer.
///////////////////////////////////////////////////////////////////////////////
cbuffer CbLight : register(b1)
{
    float3      LightPosition       : packoffset(c0);
    float       LightAngleOffset    : packoffset(c0.w);
    float3      LightColor          : packoffset(c1);
    float       LightIntensity      : packoffset(c1.w);
    float3      LightForward        : packoffset(c2);
    float       LightAngleScale     : packoffset(c2.w);
};

///////////////////////////////////////////////////////////////////////////////
// CameraBuffer
///////////////////////////////////////////////////////////////////////////////
cbuffer CbCamera : register(b2)
{
    float3 CameraPosition : packoffset(c0);     // Jʒuł.
}

//-----------------------------------------------------------------------------
// Textures and Samplers
//-----------------------------------------------------------------------------
SamplerState    IESSmp       : register(s0);
Texture2D       IESMap       : register(t0);

Texture2D       BaseColorMap : register(t1);
SamplerState    BaseColorSmp : register(s1);

Texture2D       MetallicMap  : register(t2);
SamplerState    MetallicSmp  : register(s2);

Texture2D       RoughnessMap : register(t3);
SamplerState    RoughnessSmp : register(s3);

Texture2D       NormalMap    : register(t4);
SamplerState    NormalSmp    : register(s4);


//-----------------------------------------------------------------------------
//      Ɋ炩Ɍ܂.
//-----------------------------------------------------------------------------
float SmoothDistanceAttenuation
(
    float squaredDistance,      // Cgւ̋2.
    float invSqrAttRadius       // Cga2̋t.
)
{
    float  factor = squaredDistance * invSqrAttRadius;
    float  smoothFactor = saturate(1.0f - factor * factor);
    return smoothFactor * smoothFactor;
}

//-----------------------------------------------------------------------------
//      ߂܂.
//-----------------------------------------------------------------------------
float GetDistanceAttenuation
(
    float3  unnormalizedLightVector,    // CgʒuƃIuWFNgʒu̍xNg.
    float   invSqrAttRadius             // Cga2̋t.
)
{
    float sqrDist = dot(unnormalizedLightVector, unnormalizedLightVector);
    float attenuation = 1.0f / (max(sqrDist, MIN_DIST * MIN_DIST));
    attenuation *= SmoothDistanceAttenuation(sqrDist, invSqrAttRadius);

    return attenuation;
}

//-----------------------------------------------------------------------------
//      px߂܂.
//-----------------------------------------------------------------------------
float GetAngleAttenuation
(
    float3  unnormalizedLightVector,    // CgʒuƃIuWFNgʒu̍xNg.
    float3  lightDir,                   // Kς݂̃CgxNg.
    float   lightAngleScale,            // X|bgCg̊pxXP[.
    float   lightAngleOffset            // X|bgCg̊pxItZbg.
)
{
    // CPUŎ̒lvZĂ.
    // float lightAngleScale = 1.0f / max(0.001f, (cosInner - cosOuter));
    // float lightAngleOffset  = -cosOuter * lightAngleScale;
    // Lɂ
    // * cosInner͓p̗].
    // * cosOuter͊Op̗].
    // Ƃ܂.

    float cd = dot(lightDir, unnormalizedLightVector);
    float attenuation = saturate(cd * lightAngleScale + lightAngleOffset);

    // 炩ɕω.
    attenuation *= attenuation;

    return attenuation;
}

//-----------------------------------------------------------------------------
//      IESvt@Cɂ錸߂܂.
//-----------------------------------------------------------------------------
float GetIESProfileAttenuation(float3 lightDir, float3 lightForward)
{
    // px߂.
    float  thetaCoord   = dot(lightDir, lightForward) * 0.5f + 0.5f;
    float  tangentAngle = atan2(lightDir.y, lightDir.x);
    float  phiCoord     = (tangentAngle / F_PI) * 0.5f + 0.5;
    float2 texCoord     = float2(thetaCoord, phiCoord);

    // IESeNX`琳Kꂽxo.
    return IESMap.SampleLevel(IESSmp, texCoord, 0).r;
}

//-----------------------------------------------------------------------------
//      tHggbNCg]܂.
//-----------------------------------------------------------------------------
float3 EvaluatePhotometricLight
(
    float3      N,                  // @xNg.
    float3      worldPos,           // [hԂ̃IuWFNgʒu.
    float3      lightPos,           // Cgʒu.
    float3      lightForward,       // Cg̏Ǝ˕.
    float3      lightColor,         // CgJ[.
    float       lightAngleScale,    // Cg̊pxXP[.
    float       lightAngleOffset    // Cg̊pxItZbg.
)
{
    float3 unnormalizedLightVector = (lightPos - worldPos);
    float3 L = normalize(unnormalizedLightVector);
    float att = 1.0f;
    att *= GetAngleAttenuation(-L, lightForward, lightAngleScale, lightAngleOffset);
    att *= GetIESProfileAttenuation(L, lightForward);

    return saturate(dot(N, L)) * lightColor * att / (4.0f * F_PI);
}

//-----------------------------------------------------------------------------
//      sNZVF[_̃CGg[|Cgł.
//-----------------------------------------------------------------------------
PSOutput main(VSOutput input)
{
    PSOutput output = (PSOutput)0;

    float3 L = normalize(LightPosition  - input.WorldPos);
    float3 V = normalize(CameraPosition - input.WorldPos);
    float3 H = normalize(V + L);
    float3 N = NormalMap.Sample(NormalSmp, input.TexCoord).xyz * 2.0f - 1.0f;
    N = mul(input.InvTangentBasis, N);

    float NV = saturate(dot(N, V));
    float NH = saturate(dot(N, H));
    float NL = saturate(dot(N, L));

    float3 baseColor = BaseColorMap.Sample(BaseColorSmp, input.TexCoord).rgb;
    float  metallic  = MetallicMap .Sample(MetallicSmp,  input.TexCoord).r;
    float  roughness = RoughnessMap.Sample(RoughnessSmp, input.TexCoord).r;

    float3 Kd      = baseColor * (1.0f - metallic);
    float3 diffuse = ComputeLambert(Kd);

    float3 Ks       = baseColor * metallic;
    float3 specular = ComputeGGX(Ks, roughness, NH, NV, NL);

    float3 BRDF = (diffuse + specular);
    float3 lit  = EvaluatePhotometricLight(
        N,
        input.WorldPos,
        LightPosition,
        LightForward,
        LightColor,
        LightAngleScale,
        LightAngleOffset) * LightIntensity;

    output.Color.rgb = lit * BRDF;
    output.Color.a   = 1.0f;

    return output;
}