//-----------------------------------------------------------------------------
// File : BasicVS.hlsl
// Desc : Vertex Shader.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// VSInput structure
///////////////////////////////////////////////////////////////////////////////
struct VSInput
{
    float3  Position : POSITION;    // ʒuWł.
    float3  Normal   : NORMAL;      // @xNgł.
    float2  TexCoord : TEXCOORD;    // eNX`Wł.
    float3  Tangent  : TANGENT;     // ڐxNgł.
};

///////////////////////////////////////////////////////////////////////////////
// VSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct VSOutput
{
    float4      Position        : SV_POSITION;          // ʒuWł.
    float2      TexCoord        : TEXCOORD;             // eNX`Wł.
    float3      WorldPos        : WORLD_POS;            // [hԂ̈ʒuWł.
    float3x3    InvTangentBasis : INV_TANGENT_BASIS;    // ڐԂւ̊ϊs̋tsł.
};

///////////////////////////////////////////////////////////////////////////////
// CbTransform constant buffer
///////////////////////////////////////////////////////////////////////////////
cbuffer CbTransform : register(b0)
{
    float4x4 View  : packoffset(c0); // r[sł.
    float4x4 Proj  : packoffset(c4); // ˉesł.
};

///////////////////////////////////////////////////////////////////////////////
// CbMesh constant buffer
///////////////////////////////////////////////////////////////////////////////
cbuffer CbMesh : register(b1)
{
    float4x4 World : packoffset(c0); // [hsł.
};

//-----------------------------------------------------------------------------
//      _VF[_̃CGg[|Cgł.
//-----------------------------------------------------------------------------
VSOutput main(VSInput input)
{
    VSOutput output = (VSOutput)0;

    float4 localPos = float4(input.Position, 1.0f);
    float4 worldPos = mul(World, localPos);
    float4 viewPos  = mul(View,  worldPos);
    float4 projPos  = mul(Proj,  viewPos);

    output.Position = projPos;
    output.TexCoord = input.TexCoord;
    output.WorldPos = worldPos.xyz;

    // xNg
    float3 N = normalize(mul((float3x3)World, input.Normal));
    float3 T = normalize(mul((float3x3)World, input.Tangent));
    float3 B = normalize(cross(N, T));

    // ϊs̋ts.
    output.InvTangentBasis = transpose(float3x3(T, B, N));

    return output;
}