﻿//-----------------------------------------------------------------------------
// File : SkyBox.h
// Desc : Sky Box.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------
#pragma once

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <d3d12.h>
#include <DescriptorPool.h>
#include <ConstantBuffer.h>
#include <VertexBuffer.h>
#include <SimpleMath.h>


///////////////////////////////////////////////////////////////////////////////
// SkyBox class
///////////////////////////////////////////////////////////////////////////////
class SkyBox
{
    //=========================================================================
    // list of friend classes and methods.
    //=========================================================================
    /* NOTHING */

public:
    //=========================================================================
    // public variables.
    //=========================================================================
    /* NOTHING */

    //=========================================================================
    // public methods.
    //=========================================================================

    //-------------------------------------------------------------------------
    //! @brief      コンストラクタです.
    //-------------------------------------------------------------------------
    SkyBox();

    //-------------------------------------------------------------------------
    //! @brief      デストラクタです.
    //-------------------------------------------------------------------------
    ~SkyBox();

    //-------------------------------------------------------------------------
    //! @brief      初期化処理を行います.
    //!
    //! @param[in]      pDevice         デバイスです.
    //! @param[in]      pPoolRes        CBV用ディスクリプタプールです.
    //! @param[in]      colorFormat     カラーフォーマットです.
    //! @param[in]      depthFormat     深度フォーマットです.
    //! @retval true    初期化に成功.
    //! @retval false   初期化に失敗.
    //-------------------------------------------------------------------------
    bool Init(
        ID3D12Device*   pDevice,
        DescriptorPool* pPoolRes,
        DXGI_FORMAT     colorFormat,
        DXGI_FORMAT     depthFormat);

    //-------------------------------------------------------------------------
    //! @brief      終了処理を行います.
    //-------------------------------------------------------------------------
    void Term();

    //-------------------------------------------------------------------------
    //! @brief      描画処理を行います.
    //!
    //! @param[in]      pCmd            グラフィックスコマンドリストです.
    //! @param[in]      handleCubeMap   キューブマップのディスクリプタハンドルです.
    //! @param[in]      viewMatrix      ビュー行列です.
    //! @param[in]      projMatrix      射影行列です.
    //! @param[in]      boxSize         ボックスサイズです.
    //-------------------------------------------------------------------------
    void Draw(
        ID3D12GraphicsCommandList*              pCmd,
        D3D12_GPU_DESCRIPTOR_HANDLE             handleCubeMap,
        const DirectX::SimpleMath::Matrix&      viewMatrix,
        const DirectX::SimpleMath::Matrix&      projMatrix,
        float                                   boxSize);

private:
    //=========================================================================
    // private variables.
    //=========================================================================
    DescriptorPool*                 m_pPoolRes;     //!< CBV用ディスクリプタプールです.
    ComPtr<ID3D12RootSignature>     m_pRootSig;     //!< ルートシグニチャ.
    ComPtr<ID3D12PipelineState>     m_pPSO;         //!< パイプラインステートです.
    ConstantBuffer                  m_CB[2];        //!< 定数バッファです.
    VertexBuffer                    m_VB;           //!< 頂点バッファです.
    int                             m_Index;        //!< バッファ番号です.

    //=========================================================================
    // private methods.
    //=========================================================================
    /* NOTHING */
};
