//-----------------------------------------------------------------------------
// File : BasicMS.hlsl
// Desc : Vertex Shader.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// MSInput structure
///////////////////////////////////////////////////////////////////////////////
struct MSInput
{
    float3  Position;  // ʒuWł.
    float3  Normal;    // @xNgł.
    float2  TexCoord;  // eNX`Wł.
    float3  Tangent;   // ڐxNgł.
};

///////////////////////////////////////////////////////////////////////////////
// MSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct MSOutput
{
    float4      Position        : SV_POSITION;  // ʒuWł.
    float2      TexCoord        : TEXCOORD;     // eNX`Wł.
    float3      WorldPos        : WORLD_POS;    // [hԂ̈ʒuWł.
    float3      Tangent         : TANGENT;      // ڐxNgł.
    float3      Bitangent       : BITANGENT;    // ]ڐxNgł.
    float3      Normal          : NORMAL;       // @xNgł.
};

///////////////////////////////////////////////////////////////////////////////
// Transform structure
///////////////////////////////////////////////////////////////////////////////
struct TransformParam
{
    float4x4 View;  // r[sł.
    float4x4 Proj;  // ˉesł.
};

///////////////////////////////////////////////////////////////////////////////
// MeshParam structure
///////////////////////////////////////////////////////////////////////////////
struct MeshParam
{
    float4x4 World; // [hsł.
};

///////////////////////////////////////////////////////////////////////////////
// Meshlet structure
///////////////////////////////////////////////////////////////////////////////
struct Meshlet
{
    uint VertexOffset;      // _ԍItZbg.
    uint VertexCount;       // o͒_.
    uint PrimitiveOffset;   // v~eBuԍItZbg.
    uint PrimitiveCount;    // o̓v~eBu.
};

//-----------------------------------------------------------------------------
// Resources
//-----------------------------------------------------------------------------
ConstantBuffer<TransformParam> CbTransform  : register(b0);
ConstantBuffer<MeshParam>      CbMesh       : register(b1);
StructuredBuffer<MSInput>      Vertices     : register(t0);
StructuredBuffer<uint>         Indices      : register(t1);
StructuredBuffer<Meshlet>      Meshlets     : register(t2);
StructuredBuffer<uint>         Primitives   : register(t3);


//------------------------------------------------------------------------------
//      pbLOꂽCfbNXf[^WJ.
//------------------------------------------------------------------------------
uint3 UnpackPrimitiveIndex(uint packedIndex)
{
    return uint3(
        packedIndex & 0x3FF,
        (packedIndex >> 10) & 0x3FF,
        (packedIndex >> 20) & 0x3FF);
}

//-----------------------------------------------------------------------------
//      CGg[|Cgł.
//-----------------------------------------------------------------------------
[numthreads(128, 1, 1)]
[outputtopology("triangle")]
void main
(
    uint groupThreadId  : SV_GroupThreadID,
    uint groupId        : SV_GroupID,
    out vertices MSOutput   verts[64],
    out indices  uint3      polys[126]
)
{
    Meshlet m = Meshlets[groupId];

    SetMeshOutputCounts(m.VertexCount, m.PrimitiveCount);

    if (groupThreadId < m.PrimitiveCount)
    {
        uint packedIndex     = Primitives[m.PrimitiveOffset + groupThreadId];
        polys[groupThreadId] = UnpackPrimitiveIndex(packedIndex);
    }
    
    if (groupThreadId < m.VertexCount)
    {
        uint     index  = Indices[m.VertexOffset + groupThreadId];
        MSInput  input  = Vertices[index];
        MSOutput output = (MSOutput)0;

        float4 localPos = float4(input.Position, 1.0f);
        float4 worldPos = mul(CbMesh.World,     localPos);
        float4 viewPos  = mul(CbTransform.View, worldPos);
        float4 projPos  = mul(CbTransform.Proj, viewPos);

        output.Position = projPos;
        output.TexCoord = input.TexCoord;
        output.WorldPos = worldPos.xyz;

        // xNg.
        float3 N = normalize(mul((float3x3)CbMesh.World, input.Normal));
        float3 T = normalize(mul((float3x3)CbMesh.World, input.Tangent));
        float3 B = normalize(cross(N, T));

        output.Tangent      = T;
        output.Bitangent    = B;
        output.Normal       = N;

        verts[groupThreadId] = output;
    }
}