//-----------------------------------------------------------------------------
// File : BasicPS.hlsl
// Desc : Pixel Shader.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "BRDF.hlsli"


///////////////////////////////////////////////////////////////////////////////
// MSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct MSOutput
{
    float4  Position    : SV_POSITION;  // ʒuWł.
    float2  TexCoord    : TEXCOORD;     // eNX`Wł.
    float3  WorldPos    : WORLD_POS;    // [hԂ̈ʒuWł.
    float3  Tangent     : TANGENT;      // ڐxNgł.
    float3  Bitangent   : BITANGENT;    // ]ڐxNgł.
    float3  Normal      : NORMAL;       // @xNgł.    
};

///////////////////////////////////////////////////////////////////////////////
// PSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct PSOutput
{
    float4  Color : SV_TARGET0;     // o̓J[ł.
};

///////////////////////////////////////////////////////////////////////////////
// CbLight constant buffer.
///////////////////////////////////////////////////////////////////////////////
cbuffer CbLight : register(b1)
{
    float   TextureSize     : packoffset(c0);   // eNX`TCYł.
    float   MipCount        : packoffset(c0.y); // ~bvJEgł.
    float   LightIntensity  : packoffset(c0.z); // Cgx(XP[l).
    float3  LightDirection  : packoffset(c1);   // fBNViCg̕.
};

///////////////////////////////////////////////////////////////////////////////
// CbCamera constant buffer.
///////////////////////////////////////////////////////////////////////////////
cbuffer CbCamera : register(b2)
{
    float3 CameraPosition : packoffset(c0);     // Jʒuł.
}

//-----------------------------------------------------------------------------
// Textures and Samplers
//-----------------------------------------------------------------------------
// DFG.
Texture2D    DFGMap         : register(t0);
SamplerState DFGSmp         : register(s0);

// fBt[YLD.
TextureCube  DiffuseLDMap   : register(t1);
SamplerState DiffuseLDSmp   : register(s1);

// XyL[LD.
TextureCube  SpecularLDMap  : register(t2);
SamplerState SpecularLDSmp  : register(s2);

// x[XJ[}bv.
Texture2D    BaseColorMap    : register(t3);
SamplerState BaseColorSmp    : register(s3);

// ^bN}bv.
Texture2D    MetallicMap     : register(t4);
SamplerState MetallicSmp     : register(s4);

// tlX}bv.
Texture2D    RoughnessMap    : register(t5);
SamplerState RoughnessSmp    : register(s5);

// @}bv.
Texture2D    NormalMap       : register(t6);
SamplerState NormalSmp       : register(s6);


//-----------------------------------------------------------------------------
//      XyL[̎xzIȕ߂܂.
//-----------------------------------------------------------------------------
float3 GetSpecularDomiantDir(float3 N, float3 R, float roughness)
{
    float smoothness = saturate(1.0f - roughness);
    float lerpFactor = smoothness * (sqrt(smoothness) + roughness);
    return lerp(N, R, lerpFactor);
}

//-----------------------------------------------------------------------------
//      fBt[YIBL]܂.
//-----------------------------------------------------------------------------
float3 EvaluateIBLDiffuse(float3 N)
{
    // Lambert BRDFDFG͐ϕ1.0ƂȂ̂ŁCLD݂̂ԋpΗǂ
    return DiffuseLDMap.Sample(DiffuseLDSmp, N).rgb;
}

//-----------------------------------------------------------------------------
//      `tlX~bvx߂܂.
//-----------------------------------------------------------------------------
float RoughnessToMipLevel(float linearRoughness, float mipCount)
{
    return (mipCount - 1) * linearRoughness;
}

//-----------------------------------------------------------------------------
//      XyL[IBL]܂.
//-----------------------------------------------------------------------------
float3 EvaluateIBLSpecular
(
    float           NdotV,          // @xNgƎxNg̓.
    float3          N,              // @xNg.
    float3          R,              // ˃xNg.
    float3          f0,             // tl
    float           roughness,      // `tlX.
    float           textureSize,    // eNX`TCY.
    float           mipCount        // ~bvx.
)
{
    float  a = roughness * roughness;
    float3 dominantR = GetSpecularDomiantDir(N, R, a);

    // ֐č\z.
    // L * D * (f0 * Gvis * (1 - Fc) + Gvis * Fc) * cosTheta / (4 * NdotL * NdotV).
    NdotV = max(NdotV, 0.5f / textureSize); // [ZȂ悤ɂ.
    float  mipLevel = RoughnessToMipLevel(roughness, mipCount);
    float3 preLD    = SpecularLDMap.SampleLevel(SpecularLDSmp, dominantR, mipLevel).xyz;
    
    // OϕDFGTv.
    // Fc = ( 1 - HdotL )^5
    // PreIntegratedDFG.r = Gvis * (1 - Fc)
    // PreIntegratedDFG.g = Gvis * Fc
    float2 preDFG   = DFGMap.SampleLevel(DFGSmp, float2(NdotV, roughness), 0).xy;

    // LD * (f0 * Gvis * (1 - Fc) + Gvis * Fc)
    return preLD * (f0 * preDFG.x + preDFG.y);
}


//-----------------------------------------------------------------------------
//      sNZVF[_̃CGg[|Cgł.
//-----------------------------------------------------------------------------
PSOutput main(MSOutput input)
{
    PSOutput output = (PSOutput)0;
    
    float3x3 invTangentBasis = transpose(float3x3(input.Tangent, input.Bitangent, input.Normal));

    float3 V = normalize(input.WorldPos.xyz - CameraPosition);
    float3 N = NormalMap.Sample(NormalSmp, input.TexCoord).xyz * 2.0f - 1.0f;
    N = mul(invTangentBasis, N);
    float3 R = normalize(reflect(V, N));

    float NV = saturate(dot(N, V));

    float3 baseColor = BaseColorMap.Sample(BaseColorSmp, input.TexCoord).rgb;
    float  metallic  = MetallicMap .Sample(MetallicSmp,  input.TexCoord).r;
    float  roughness = RoughnessMap.Sample(RoughnessSmp, input.TexCoord).r;

    float3 Kd = baseColor * (1.0f - metallic);
    float3 Ks = baseColor * metallic;

    float3 lit = 0;
    lit += EvaluateIBLDiffuse(N) * Kd;
    lit += EvaluateIBLSpecular(NV, N, R, Ks, roughness, TextureSize, MipCount);

    output.Color.rgb = lit * LightIntensity;
    output.Color.a   = 1.0f;

    return output;
}