﻿//-----------------------------------------------------------------------------
// File : Mesh.h
// Desc : Mesh Drawer Module.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------
#pragma once

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <ResMesh.h>
#include <VertexBuffer.h>
#include <IndexBuffer.h>


///////////////////////////////////////////////////////////////////////////////
// Mesh class
///////////////////////////////////////////////////////////////////////////////
class Mesh
{
    //=========================================================================
    // list of friend classes and methods.
    //=========================================================================
    /* NOTHING */

public:
    //=========================================================================
    // public variables.
    //=========================================================================
    /* NOTHING */

    //=========================================================================
    // public methods.
    //=========================================================================

    //-------------------------------------------------------------------------
    //! @brief      コンストラクタです.
    //-------------------------------------------------------------------------
    Mesh();

    //-------------------------------------------------------------------------
    //! @brief      デストラクタです.
    //-------------------------------------------------------------------------
    virtual ~Mesh();

    //-------------------------------------------------------------------------
    //! @brief      初期化処理を行います.
    //!
    //! @param[in]      pDevice         デバイスです.
    //! @param[in]      resource        リソースメッシュです.
    //! @retval true    初期化に成功.
    //! @retval false   初期化に失敗.
    //-------------------------------------------------------------------------
    bool Init(ID3D12Device* pDevice, const ResMesh& resource);

    //-------------------------------------------------------------------------
    //! @brief      終了処理を行います.
    //-------------------------------------------------------------------------
    void Term();

    //-------------------------------------------------------------------------
    //! @brief      描画処理を行います.
    //!
    //! @param[in]      pCmdList        コマンドリストです.
    //-------------------------------------------------------------------------
    void Draw(ID3D12GraphicsCommandList* pCmdList);

    //-------------------------------------------------------------------------
    //! @brief      マテリアルIDを取得します.
    //!
    //! @return     マテリアルIDを返却します.
    //-------------------------------------------------------------------------
    uint32_t GetMaterialId() const;

private:
    //=========================================================================
    // private variables.
    //=========================================================================
    VertexBuffer    m_VB;               //!< 頂点バッファです.
    IndexBuffer     m_IB;               //!< インデックスバッファです.
    uint32_t        m_MaterialId;       //!< マテリアルIDです.
    uint32_t        m_IndexCount;       //!< インデックス数です.

    //=========================================================================
    // private methods.
    //=========================================================================
    Mesh            (const Mesh&) = delete;     // アクセス禁止.
    void operator = (const Mesh&) = delete;     // アクセス禁止.
};