//-----------------------------------------------------------------------------
// File : PhongPS.hlsl
// Desc : Phong Lighting Pixel Shader.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// VSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct VSOutput
{
    float4      Position        : SV_POSITION;          // ʒuWł.
    float2      TexCoord        : TEXCOORD;             // eNX`Wł.
    float4      WorldPos        : WORLD_POS;            // [hԂł̈ʒuWł.
#if 0
    float3x3    TangentBasis    : TANGENT_BASIS;        // ڐԂւ̊ϊsł.
#else
    float3x3    InvTangentBasis : INV_TANGENT_BASIS;    // ڐԂւ̊ϊs̋tsł.
#endif
};

///////////////////////////////////////////////////////////////////////////////
// PSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct PSOutput
{
    float4  Color : SV_TARGET0;     // o̓J[ł.
};

///////////////////////////////////////////////////////////////////////////////
// LightBuffer 
///////////////////////////////////////////////////////////////////////////////
cbuffer LightBuffer : register( b1 )
{
    float3 LightPosition  : packoffset( c0 );   // Cg̈ʒuWł.
    float3 LightColor     : packoffset( c1 );   // CgJ[ł.
    float3 CameraPosition : packoffset( c2 );   // J̈ʒuWł.
};

///////////////////////////////////////////////////////////////////////////////
// MaterialBuffer
///////////////////////////////////////////////////////////////////////////////
cbuffer MaterialBuffer : register( b2 )
{
    float3 Diffuse      : packoffset( c0 );         // gU˗ł.
    float  Alpha        : packoffset( c0.w );       // ߓxł.
    float3 Specular     : packoffset( c1 );         // ʔ˗ł.
    float  Shininess    : packoffset( c1.w );       // ʔˋxł.
};

//-----------------------------------------------------------------------------
// Textures and Samplers
//-----------------------------------------------------------------------------
SamplerState WrapSmp    : register( s0 );       // eNX`s[g
Texture2D    ColorMap   : register( t0 );       // J[}bvł.
Texture2D    NormalMap  : register( t1 );       // @}bvł.

//-----------------------------------------------------------------------------
//      sNZVF[_̃CGg[|Cgł.
//-----------------------------------------------------------------------------
PSOutput main(VSOutput input)
{
    PSOutput output = (PSOutput)0;

#if 0
    /* ڐԏŃCeBOꍇ */

    // CgxNg.
    float3 L = normalize(LightPosition - input.WorldPos.xyz);
    L = mul(input.TangentBasis, L);

    // xNg.
    float3 V = normalize(CameraPosition - input.WorldPos.xyz);
    V = mul(input.TangentBasis, V);

    // @xNg.
    float3 N = NormalMap.Sample(WrapSmp, input.TexCoord).xyz * 2.0 - 1.0;
#else
    /* [hԏŃCeBOꍇ */

    // CgxNg.
    float3 L = normalize(LightPosition - input.WorldPos.xyz);

    // xNg.
    float3 V = normalize(CameraPosition - input.WorldPos.xyz);

    // @xNg.
    float3 N = NormalMap.Sample(WrapSmp, input.TexCoord).xyz * 2.0 - 1.0;
    N = mul(input.InvTangentBasis, N);
#endif

    // ˃xNg.
    float3 R = normalize(-reflect(V, N));

    float4 color    = ColorMap.Sample( WrapSmp, input.TexCoord );
    float3 diffuse  = Diffuse  * LightColor * saturate(dot(L, N));
    float3 specular = Specular * LightColor * pow( saturate(dot(L, R)), Shininess );

    output.Color = float4( color.rgb * ( diffuse + specular ), color.a  * Alpha );

    return output;
}