//-----------------------------------------------------------------------------
// File : PhongVS.hlsl
// Desc : Phong Lighting Vertex Shader.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// VSInput structure
///////////////////////////////////////////////////////////////////////////////
struct VSInput
{
    float3  Position : POSITION;    // ʒuWł.
    float3  Normal   : NORMAL;      // @xNgł.
    float2  TexCoord : TEXCOORD;    // eNX`Wł.
    float3  Tangent  : TANGENT;     // ڐxNgł.
};

///////////////////////////////////////////////////////////////////////////////
// VSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct VSOutput
{
    float4      Position        : SV_POSITION;          // ʒuWł.
    float2      TexCoord        : TEXCOORD;             // eNX`Wł.
    float4      WorldPos        : WORLD_POS;            // [hԂł̈ʒuWł.
#if 0
    float3x3    TangentBasis    : TANGENT_BASIS;        // ڐԂւ̊ϊsł.
#else
    float3x3    InvTangentBasis : INV_TANGENT_BASIS;    // ڐԂւ̊ϊs̋tsł.
#endif
};

///////////////////////////////////////////////////////////////////////////////
// Transform constant buffer
///////////////////////////////////////////////////////////////////////////////
cbuffer Transform : register( b0 )
{
    float4x4 World : packoffset( c0 ); // [hsł.
    float4x4 View  : packoffset( c4 ); // r[sł.
    float4x4 Proj  : packoffset( c8 ); // ˉesł.
}

//-----------------------------------------------------------------------------
//      _VF[_̃CGg[|Cgł.
//-----------------------------------------------------------------------------
VSOutput main( VSInput input )
{
    VSOutput output = (VSOutput)0;

    float4 localPos = float4( input.Position, 1.0f );
    float4 worldPos = mul( World, localPos );
    float4 viewPos  = mul( View,  worldPos );
    float4 projPos  = mul( Proj,  viewPos );

    output.Position = projPos;
    output.TexCoord = input.TexCoord;
    output.WorldPos = worldPos;

    // xNg
    float3 N = normalize(mul((float3x3)World, input.Normal));
    float3 T = normalize(mul((float3x3)World, input.Tangent));
    float3 B = normalize(cross(N, T));

#if 0
    /* ڐԏŃCeBOꍇ */

    // ϊs.
    output.TangentBasis    = float3x3(T, B, N);

#else
    /* [hԏŃCeBOꍇ */

    // ϊs̋ts.
    output.InvTangentBasis = transpose(float3x3(T, B, N));
#endif

    return output;
}