﻿//-----------------------------------------------------------------------------
// File : CommandList.h
// Desc : Command List Moudle.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------
#pragma once

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <d3d12.h>
#include <ComPtr.h>
#include <cstdint>
#include <vector>


///////////////////////////////////////////////////////////////////////////////
// CommandList class
///////////////////////////////////////////////////////////////////////////////
class CommandList
{
    //=========================================================================
    // list of friend classes and methods.
    //=========================================================================
    /* NOTHING */

public:
    //=========================================================================
    // public variables.
    //=========================================================================
    /* NOTHING */

    //=========================================================================
    // public methods.
    //=========================================================================

    //-------------------------------------------------------------------------
    //! @brief      コンストラクタです.
    //-------------------------------------------------------------------------
    CommandList();

    //-------------------------------------------------------------------------
    //! @brief      デストラクタです.
    //-------------------------------------------------------------------------
    ~CommandList();

    //-------------------------------------------------------------------------
    //! @brief      初期化処理を行います.
    //!
    //! @param[in]      pDevice     デバイスです.
    //! @param[in]      type        コマンドリストタイプです.
    //! @param[in]      count       アロケータの数です.ダブルバッファ化する場合は2に設定します.
    //! @retval true    初期化に成功.
    //! @retval fasle   初期化に失敗.
    //-------------------------------------------------------------------------
    bool Init(ID3D12Device* pDevice, D3D12_COMMAND_LIST_TYPE type, uint32_t count);

    //-------------------------------------------------------------------------
    //! @brief      終了処理を行います.
    //-------------------------------------------------------------------------
    void Term();

    //-------------------------------------------------------------------------
    //! @brief      リセット処理を行ったコマンドリストを取得します.
    //!
    //! @return     リセット処理を行ったコマンドリストを返却します.
    //-------------------------------------------------------------------------
    ID3D12GraphicsCommandList* Reset();

private:
    //=========================================================================
    // private variables.
    //=========================================================================
    ComPtr<ID3D12GraphicsCommandList>           m_pCmdList;         //!< コマンドリストです.
    std::vector<ComPtr<ID3D12CommandAllocator>> m_pAllocators;      //!< コマンドアロケータです.
    uint32_t                                    m_Index;            //!< アロケータ番号です.

    //=========================================================================
    // private methods.
    //=========================================================================
    CommandList     (const CommandList&) = delete;
    void operator = (const CommandList&) = delete;
};
