//-----------------------------------------------------------------------------
// File : SimplePS.hlsl
// Desc : Simple Pixel Shader.
// Copyright(c) Pocol. All right reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Constant Values
//-----------------------------------------------------------------------------
static const float F_PI = 3.141596535f;


///////////////////////////////////////////////////////////////////////////////
// VSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct VSOutput
{
    float4  Position : SV_POSITION;     // ʒuWł.
    float2  TexCoord : TEXCOORD;        // eNX`Wł.
    float3  Normal   : NORMAL;          // @xNgł.
    float4  WorldPos : WORLD_POS;       // [hԂł̈ʒuWł.
};

///////////////////////////////////////////////////////////////////////////////
// PSOutput structure
///////////////////////////////////////////////////////////////////////////////
struct PSOutput
{
    float4  Color : SV_TARGET0;     // o̓J[ł.
};

///////////////////////////////////////////////////////////////////////////////
// LightBuffer 
///////////////////////////////////////////////////////////////////////////////
cbuffer LightBuffer : register( b1 )
{
    float3 LightPosition : packoffset( c0 );    // Cgʒuł.
    float3 LightColor    : packoffset( c1 );    // CgJ[ł.
}

///////////////////////////////////////////////////////////////////////////////
// MaterialBuffer
///////////////////////////////////////////////////////////////////////////////
cbuffer MaterialBuffer : register(b2)
{
    float3 Diffuse : packoffset( c0 );      // gU˗ł.
    float  Alpha   : packoffset( c0.w );    // ߓxł.
}

//-----------------------------------------------------------------------------
// Textures and Samplers
//-----------------------------------------------------------------------------
SamplerState WrapSmp  : register( s0 );
Texture2D    ColorMap : register( t0 );

//-----------------------------------------------------------------------------
//      sNZVF[_̃CGg[|Cgł.
//-----------------------------------------------------------------------------
PSOutput main(VSOutput input)
{
    PSOutput output = (PSOutput)0;

    float3 N = normalize(input.Normal);
    float3 L = normalize(LightPosition - input.WorldPos.xyz);

    float NL = saturate(dot(N, L));

    float4 color   = ColorMap.Sample( WrapSmp, input.TexCoord );
    float3 diffuse = Diffuse * (1.0 / F_PI);

    output.Color = float4( LightColor * color.rgb * diffuse * NL, color.a * Alpha );

    return output;
}
